@@ -22,6 +22,7 @@
 #include "../Effects/EffectMgr.h"
 #include "../DSP/FFTwrapper.h"
 #include "../Misc/Allocator.h"
+#include "../Containers/ScratchString.h"
 #include "../Nio/Nio.h"
 #include "PresetExtractor.h"
 
@@ -74,7 +75,7 @@ static const Ports sysefxPort =
 
 static const Ports sysefsendto =
 {
-    {"to#" STRINGIFY(NUM_SYS_EFX) "::i", 
+    {"to#" STRINGIFY(NUM_SYS_EFX) "::i",
         rProp(parameter) rDoc("sysefx to sysefx routing gain"), 0, [](const char *m, RtData&d)
         {
             //same ugly workaround as before
@@ -97,6 +98,17 @@ static const Ports sysefsendto =
         }}
 };
 
+#define rBegin [](const char *msg, RtData &d) { Master *m = (Master*)d.obj
+#define rEnd }
+
+static const Ports watchPorts = {
+    {"add:s", rDoc("Add synthesis state to watch"), 0,
+        rBegin;
+        m->watcher.add_watch(rtosc_argument(msg,0).s);
+        rEnd},
+};
+
+extern const Ports bankPorts;
 static const Ports master_ports = {
     rString(last_xmz, XMZ_PATH_MAX, "File name for last name loaded if any."),
     rRecursp(part, 16, "Part"),//NUM_MIDI_PARTS
@@ -104,8 +116,12 @@ static const Ports master_ports = {
     rRecursp(insefx, 8, "Insertion Effect"),//NUM_INS_EFX
     rRecur(microtonal, "Micrtonal Mapping Functionality"),
     rRecur(ctl, "Controller"),
-    rArrayI(Pinsparts, NUM_INS_EFX, "Part to insert part onto"),
-    {"Pkeyshift::i", rProp(parameter) rLinear(0,127) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
+    rArrayI(Pinsparts, NUM_INS_EFX, rOpt(-2, Master), rOpt(-1, Off)
+            rOptions(Part1, Part2, Part3, Part4,  Part5, Part6,
+                 Part7, Part8, Part9, Part10, Part11, Part12,
+                 Part13, Part14, Part15, Part16),
+                "Part to insert part onto"),
+    {"Pkeyshift::i", rShort("key shift") rProp(parameter) rLinear(0,127) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pkeyshift);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
@@ -116,6 +132,22 @@ static const Ports master_ports = {
     {"get-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
        d.reply("/vu-meter", "bb", sizeof(m->vu), &m->vu, sizeof(float)*NUM_MIDI_PARTS, m->vuoutpeakpart);}},
+    {"vu-meter:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
+       Master *m = (Master*)d.obj;
+       char        types[6+NUM_MIDI_PARTS+1] = {0};
+       rtosc_arg_t  args[6+NUM_MIDI_PARTS+1];
+       for(int i=0; i<6+NUM_MIDI_PARTS; ++i)
+           types[i] = 'f';
+       args[0].f = m->vu.outpeakl;
+       args[1].f = m->vu.outpeakr;
+       args[2].f = m->vu.maxoutpeakl;
+       args[3].f = m->vu.maxoutpeakr;
+       args[4].f = m->vu.rmspeakl;
+       args[5].f = m->vu.rmspeakr;
+       for(int i=0; i<NUM_MIDI_PARTS; ++i)
+           args[6+i].f = m->vuoutpeakpart[i];
+
+       d.replyArray("/vu-meter", types, args);}},
     {"reset-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
        m->vuresetpeaks();}},
@@ -129,14 +161,21 @@ static const Ports master_ports = {
        m->part[i] = p;
        p->initialize_rt();
        }},
-    {"Pvolume::i", rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
+    {"active_keys:", rProp("Obtain a list of active notes"), 0,
+        rBegin;
+        char keys[129] = {0};
+        for(int i=0; i<128; ++i)
+            keys[i] = m->activeNotes[i] ? 'T' : 'F';
+        d.broadcast(d.loc, keys);
+        rEnd},
+    {"Pvolume::i", rShort("volume") rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
             ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
             d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
-    {"volume::i", rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
+    {"volume::i", rShort("volume") rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
@@ -240,8 +279,18 @@ static const Ports master_ports = {
     {"HDDRecorder/pause:", rDoc("Pause recording"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
        m->HDDRecorder.pause();}},
-
+    {"watch/", rDoc("Interface to grab out live synthesis state"), &watchPorts,
+        rBOIL_BEGIN;
+        SNIP;
+        watchPorts.dispatch(msg, data);
+        rBOIL_END},
+    {"bank/", rDoc("Controls for instrument banks"), &bankPorts,
+            [](const char*,RtData&) {}},
 };
+
+#undef rBegin
+#undef rEnd
+
 const Ports &Master::ports = master_ports;
 
 class DataObj:public rtosc::RtData
@@ -257,6 +306,12 @@ class DataObj:public rtosc::RtData
             forwarded = false;
         }
 
+        virtual void replyArray(const char *path, const char *args, rtosc_arg_t *vals) override
+        {
+            char *buffer = bToU->buffer();
+            rtosc_amessage(buffer,bToU->buffer_size(),path,args,vals);
+            reply(buffer);
+        }
         virtual void reply(const char *path, const char *args, ...) override
         {
             va_list va;
@@ -333,9 +388,11 @@ Master::Master(const SYNTH_T &synth_, Config* config)
         fakepeakpart[npart]  = 0;
     }
 
+    ScratchString ss;
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
         part[npart] = new Part(*memory, synth, time, config->cfg.GzipCompression,
-                               config->cfg.Interpolation, &microtonal, fft);
+                               config->cfg.Interpolation, &microtonal, fft, &watcher,
+                               (ss+"/part"+npart+"/").c_str);
 
     //Insertion Effects init
     for(int nefx = 0; nefx < NUM_INS_EFX; ++nefx)
@@ -345,6 +402,9 @@ Master::Master(const SYNTH_T &synth_, Config* config)
     for(int nefx = 0; nefx < NUM_SYS_EFX; ++nefx)
         sysefx[nefx] = new EffectMgr(*memory, synth, 0, &time);
 
+    //Note Visualization
+    for(int i=0; i<128; ++i)
+        activeNotes[i] = 0;
 
     defaults();
 
@@ -358,7 +418,7 @@ void Master::applyOscEvent(const char *msg)
     DataObj d{loc_buf, 1024, this, bToU};
     memset(loc_buf, 0, sizeof(loc_buf));
     d.matches = 0;
-        
+
     if(strcmp(msg, "/get-vu") && false) {
         fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 5 + 30, 0 + 40);
         fprintf(stdout, "backend[*]: '%s'<%s>\n", msg,
@@ -411,12 +471,14 @@ void Master::defaults()
 void Master::noteOn(char chan, char note, char velocity)
 {
     if(velocity) {
-        for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
+        for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
             if(chan == part[npart]->Prcvchn) {
                 fakepeakpart[npart] = velocity * 2;
                 if(part[npart]->Penabled)
                     part[npart]->NoteOn(note, velocity, keyshift);
             }
+        }
+        activeNotes[(int)note] = 1;
     }
     else
         this->noteOff(chan, note);
@@ -431,6 +493,7 @@ void Master::noteOff(char chan, char note)
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
         if((chan == part[npart]->Prcvchn) && part[npart]->Penabled)
             part[npart]->NoteOff(note);
+    activeNotes[(int)note] = 0;
 }
 
 /*
@@ -619,20 +682,8 @@ void dump_msg(const char* ptr, std::ostream& os = std::cerr)
 #endif
 int msg_id=0;
 
-/*
- * Master audio out (the final sound)
- */
-bool Master::AudioOut(float *outr, float *outl)
+bool Master::runOSC(float *outl, float *outr, bool offline)
 {
-    //Danger Limits
-    if(memory->lowMemory(2,1024*1024))
-        printf("QUITE LOW MEMORY IN THE RT POOL BE PREPARED FOR WEIRD BEHAVIOR!!\n");
-    //Normal Limits
-    if(!pendingMemory && memory->lowMemory(4,1024*1024)) {
-        printf("Requesting more memory\n");
-        bToU->write("/request-memory", "");
-        pendingMemory = true;
-    }
     //Handle user events TODO move me to a proper location
     char loc_buf[1024];
     DataObj d{loc_buf, 1024, this, bToU};
@@ -644,7 +695,8 @@ bool Master::AudioOut(float *outr, float *outl)
         if(!strcmp(msg, "/load-master")) {
             Master *this_master = this;
             Master *new_master  = *(Master**)rtosc_argument(msg, 0).b.data;
-            new_master->AudioOut(outl, outr);
+            if(!offline)
+                new_master->AudioOut(outl, outr);
             Nio::masterSwap(new_master);
             if (mastercb)
                 mastercb(mastercb_ptr, new_master);
@@ -661,23 +713,55 @@ bool Master::AudioOut(float *outr, float *outl)
         }
         ports.dispatch(msg, d, true);
         events++;
+        if(!d.matches) {
+            //workaround for requesting voice status
+            int a=0, b=0, c=0;
+            char e=0;
+            if(4 == sscanf(msg, "/part%d/kit%d/adpars/VoicePar%d/Enable%c", &a, &b, &c, &e)) {
+                d.reply(msg, "F");
+                d.matches++;
+            }
+        }
         if(!d.matches) {// && !ports.apropos(msg)) {
             fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 1, 7 + 30, 0 + 40);
-            fprintf(stderr, "Unknown address<BACKEND> '%s:%s'\n", uToB->peak(), rtosc_argument_string(uToB->peak()));
-#if 0
-            if(strstr(msg, "PFMVelocity"))
-                dump_msg(msg);
-            if(ports.apropos(msg))
-                fprintf(stderr, "  -> best match: '%s'\n", ports.apropos(msg)->name);
-            if(ports.apropos(msg+1))
-                fprintf(stderr, "  -> best match: '%s'\n", ports.apropos(msg+1)->name);
-#endif
+            fprintf(stderr, "Unknown address<BACKEND:%s> '%s:%s'\n", 
+                    offline ? "offline" : "online",
+                    uToB->peak(),
+                    rtosc_argument_string(uToB->peak()));
             fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
         }
     }
     if(events>1 && false)
         fprintf(stderr, "backend: %d events per cycle\n",events);
-        
+
+    return true;
+}
+
+/*
+ * Master audio out (the final sound)
+ */
+bool Master::AudioOut(float *outr, float *outl)
+{
+    //Danger Limits
+    if(memory->lowMemory(2,1024*1024))
+        printf("QUITE LOW MEMORY IN THE RT POOL BE PREPARED FOR WEIRD BEHAVIOR!!\n");
+    //Normal Limits
+    if(!pendingMemory && memory->lowMemory(4,1024*1024)) {
+        printf("Requesting more memory\n");
+        bToU->write("/request-memory", "");
+        pendingMemory = true;
+    }
+
+    //work through events
+    if(!runOSC(outl, outr, false))
+        return false;
+
+
+    //Handle watch points
+    if(bToU)
+        watcher.write_back = bToU;
+    watcher.tick();
+
 
     //Swaps the Left channel with Right Channel
     if(swaplr)
@@ -826,6 +910,17 @@ bool Master::AudioOut(float *outr, float *outl)
     //update the global frame timer
     time++;
 
+#ifdef DEMO_VERSION
+    double seconds = time.time()*synth.buffersize_f/synth.samplerate_f;
+    if(seconds > 10*60) {//10 minute trial
+        shutup = true;
+    }
+#endif
+
+    //Update pulse
+    last_ack = last_beat;
+
+
     return true;
 }
 
@@ -928,6 +1023,8 @@ void Master::ShutUp()
         insefx[nefx]->cleanup();
     for(int nefx = 0; nefx < NUM_SYS_EFX; ++nefx)
         sysefx[nefx]->cleanup();
+    for(int i = 0; i < sizeof(activeNotes)/sizeof(activeNotes[0]); ++i)
+        activeNotes[i] = 0;
     vuresetpeaks();
     shutup = 0;
 }
